@@ -5,7 +5,7 @@ use strict;
 
 use XML::Atom;
 use base qw( XML::Atom::ErrorHandler );
-use XML::Atom::Util qw( first nodelist remove_default_ns );
+use XML::Atom::Util qw( set_ns first nodelist remove_default_ns );
 use XML::Atom::Link;
 use LWP::UserAgent;
 BEGIN {
@@ -20,8 +20,6 @@ BEGIN {
     }
 }
 
-use constant NS => 'http://purl.org/atom/ns#';
-
 sub new {
     my $class = shift;
     my $atom = bless {}, $class;
@@ -29,9 +27,12 @@ sub new {
     $atom;
 }
 
+sub ns   { $_[0]->{ns} }
+
 sub init_libxml {
     my $atom = shift;
     my %param = @_ == 1 ? (Stream => $_[0]) : @_;
+    $atom->set_ns(\%param);
     if (%param) {
         if (my $stream = $param{Stream}) {
             my $parser = XML::LibXML->new;
@@ -48,17 +49,31 @@ sub init_libxml {
             $atom->{doc} = XML::LibXML::Document->createDocument('1.0', 'utf-8');
             $atom->{doc}->setDocumentElement($elem);
         }
+        if ($atom->{doc}) {
+            $atom->fixup_ns;
+        }
     } else {
         my $doc = $atom->{doc} = XML::LibXML::Document->createDocument('1.0', 'utf-8');
-        my $root = $doc->createElementNS(NS, $atom->element_name);
+        my $root = $doc->createElementNS($atom->ns, $atom->element_name);
         $doc->setDocumentElement($root);
     }
     $atom;
 }
 
+sub fixup_ns {
+    my $atom = shift;
+    $atom->{ns} = $atom->{doc}->getDocumentElement->namespaceURI;
+}
+
+sub version {
+    my $atom = shift;
+    XML::Atom::Util::ns_to_version($atom->ns);
+}
+
 sub init_xpath {
     my $atom = shift;
     my %param = @_ == 1 ? (Stream => $_[0]) : @_;
+    $atom->set_ns(\%param);
     my $elem_name = $atom->element_name;
     if (%param) {
         if (my $stream = $param{Stream}) {
@@ -87,9 +102,9 @@ sub init_xpath {
         }
     } else {
         my $xp = XML::XPath->new;
-        $xp->set_namespace(atom => NS);
+        $xp->set_namespace(atom => $atom->ns);
         $atom->{doc} = XML::XPath::Node::Element->new($atom->element_name);
-        my $ns = XML::XPath::Node::Namespace->new('#default' => NS);
+        my $ns = XML::XPath::Node::Namespace->new('#default' => $atom->ns);
         $atom->{doc}->appendNamespace($ns);
     }
     $atom;
@@ -141,7 +156,7 @@ sub set_libxml {
     }
     my $elem = $atom->{doc}->createElementNS($ns_uri, $name);
     $atom->{doc}->getDocumentElement->appendChild($elem);
-    if ($ns ne NS) {
+    if ($ns ne $atom->ns) {
         $atom->{doc}->getDocumentElement->setNamespace($ns->{uri}, $ns->{prefix}, 0);
     }
     if (ref($val) =~ /Element$/) {
@@ -168,7 +183,7 @@ sub set_xpath {
         $atom->{doc}->removeChild($_) for @elem;
     }
     my $elem = XML::XPath::Node::Element->new($name);
-    if ($ns ne NS) {
+    if ($ns ne $atom->ns) {
         my $ns = XML::XPath::Node::Namespace->new($ns->{prefix} => $ns->{uri});
         $elem->appendNamespace($ns);
     }
@@ -200,11 +215,11 @@ sub add_link {
 	}
     } else {
 	if (LIBXML) {
-	    $elem = $thing->{doc}->createElementNS(NS, 'link');
+	    $elem = $thing->{doc}->createElementNS($thing->ns, 'link');
 	    $thing->{doc}->getDocumentElement->appendChild($elem);
 	} else {
 	    $elem = XML::XPath::Node::Element->new('link');
-	    my $ns = XML::XPath::Node::Namespace->new('#default' => NS);
+	    my $ns = XML::XPath::Node::Namespace->new('#default' => $thing->ns);
 	    $elem->appendNamespace($ns);
 	    $thing->{doc}->appendChild($elem);
 	}
@@ -220,14 +235,14 @@ sub add_link {
 sub link_libxml {
     my $thing = shift;
     if (wantarray) {
-        my @res = $thing->{doc}->getDocumentElement->getChildrenByTagNameNS(NS, 'link');
+        my @res = $thing->{doc}->getDocumentElement->getChildrenByTagNameNS($thing->ns, 'link');
         my @links;
         for my $elem (@res) {
             push @links, XML::Atom::Link->new(Elem => $elem);
         }
         return @links;
     } else {
-        my $elem = first($thing->{doc}, NS, 'link') or return;
+        my $elem = first($thing->{doc}, $thing->ns, 'link') or return;
         return XML::Atom::Link->new(Elem => $elem);
     }
 }
@@ -235,14 +250,14 @@ sub link_libxml {
 sub link_xpath {
     my $thing = shift;
     if (wantarray) {
-        my $set = $thing->{doc}->find("*[local-name()='link' and namespace-uri()='" . NS . "']");
+        my $set = $thing->{doc}->find("*[local-name()='link' and namespace-uri()='" . $thing->ns . "']");
         my @links;
         for my $elem ($set->get_nodelist) {
             push @links, XML::Atom::Link->new(Elem => $elem);
         }
         return @links;
     } else {
-        my $elem = first($thing->{doc}, NS, 'link') or return;
+        my $elem = first($thing->{doc}, $thing->ns, 'link') or return;
         return XML::Atom::Link->new(Elem => $elem);
     }
 }
@@ -252,10 +267,20 @@ sub author {
     $thing->_element('XML::Atom::Person', 'author', @_);
 }
 
+sub contributor {
+    my $thing = shift;
+    $thing->_element('XML::Atom::Person', 'contributor', @_);
+}
+
 sub as_xml {
     my $doc = $_[0]->{doc};
-    remove_default_ns($doc->getDocumentElement);
-    $doc->toString(LIBXML ? 1 : 0);
+    remove_default_ns($doc->getDocumentElement) if LIBXML;
+    my $xml = $doc->toString(LIBXML ? 1 : 0);
+    if ($] > 5.008) {
+        require Encode;
+        Encode::_utf8_off($xml);
+    }
+    $xml;
 }
 
 sub _element {
@@ -263,39 +288,42 @@ sub _element {
     my($class, $name) = (shift, shift);
     my $root = LIBXML ? $thing->{doc}->getDocumentElement : $thing->{doc};
     if (@_) {
-        my $obj = shift;
-        if (my $node = first($thing->{doc}, NS, $name)) {
+        for my $node (nodelist($thing->{doc}, $thing->ns, $name)) {
             $root->removeChild($node);
         }
-        my $elem = LIBXML ?
-            $thing->{doc}->createElementNS(NS, $name) :
-            XML::XPath::Node::Element->new($name);
-        $root->appendChild($elem);
-        if (LIBXML) {
-            for my $child ($obj->elem->childNodes) {
-                $elem->appendChild($child->cloneNode(1));
-            }
-            for my $attr ($obj->elem->attributes) {
-                next unless ref($attr) eq 'XML::LibXML::Attr';
-                $elem->setAttribute($attr->getName, $attr->getValue);
-            }
-        } else {
-            for my $child ($obj->elem->getChildNodes) {
-                $elem->appendChild($child);
-            }
-            for my $attr ($obj->elem->getAttributes) {
-                $elem->appendAttribute($attr);
+        my @obj = @_;
+        for my $obj (@_) {
+            my $elem = LIBXML ?
+                $thing->{doc}->createElementNS($thing->ns, $name) :
+                    XML::XPath::Node::Element->new($name);
+            $root->appendChild($elem);
+            if (LIBXML) {
+                for my $child ($obj->elem->childNodes) {
+                    $elem->appendChild($child->cloneNode(1));
+                }
+                for my $attr ($obj->elem->attributes) {
+                    next unless ref($attr) eq 'XML::LibXML::Attr';
+                    $elem->setAttribute($attr->getName, $attr->getValue);
+                }
+            } else {
+                for my $child ($obj->elem->getChildNodes) {
+                    $elem->appendChild($child);
+                }
+                for my $attr ($obj->elem->getAttributes) {
+                    $elem->appendAttribute($attr);
+                }
             }
+            $obj->{elem} = $elem;
         }
-        $obj->{elem} = $elem;
-        $thing->{'__' . $name} = $obj;
+        $thing->{'__' . $name} = \@obj;
     } else {
         unless (exists $thing->{'__' . $name}) {
-            my $elem = first($thing->{doc}, NS, $name) or return;
-            $thing->{'__' . $name} = $class->new(Elem => $elem);
+            my @elem = nodelist($thing->{doc}, $thing->ns, $name);
+            return unless @elem;
+            $thing->{'__' . $name} = [ map $class->new(Elem => $_, Namespace => $thing->ns), @elem ];
         }
     }
-    $thing->{'__' . $name};
+    wantarray ? @{$thing->{'__' . $name}} : $thing->{'__' . $name}->[0];
 }
 
 sub DESTROY { }
@@ -305,7 +333,7 @@ sub AUTOLOAD {
     (my $var = $AUTOLOAD) =~ s!.+::!!;
     no strict 'refs';
     *$AUTOLOAD = sub {
-        @_ > 1 ? $_[0]->set(NS, $var, @_[1..$#_]) : $_[0]->get(NS, $var)
+        @_ > 1 ? $_[0]->set($_[0]->ns, $var, @_[1..$#_]) : $_[0]->get($_[0]->ns, $var)
     };
     goto &$AUTOLOAD;
 }
